# Dynamically Generated
@builtin "number.ne"
@builtin "string.ne"
@builtin "whitespace.ne"
@{%
    function compact(v) {
        if (Array.isArray(v)) {
            return v.reduce((a, v) => (v === null || v === undefined || (v && v.length === 0) ) ? a : (a.push(v), a), []);
        } else {
            return v;
        }
    }

    function flatten(v) {
        let r;
        if (Array.isArray(v)) {
            r = v.reduce((a,v) => (a.push(...((v && Array.isArray(v)) ? flatten(v) : [v])), a), []);
        } else {
            r = v;
        }
        r = compact(r);
        return r;
        }
%}


rule -> rule1 rule2 rule3 rule4 rule5 rule6 rule7 rule8 rule9 rule10 rule11 rule12 rule13 rule14 
{% ([ rule1,rule2,rule3,rule4,rule5,rule6,rule7,rule8,rule9,rule10,rule11,rule12,rule13,rule14 ]) => {
    return {
        $class: "org.accordproject.installmentsale.TemplateModel",
        
        BUYER : rule1,
        SELLER : rule3,
        INITIAL_DUE : rule5,
        INTEREST_RATE : rule13,
        TOTAL_DUE_BEFORE_CLOSING : rule9,
        MIN_PAYMENT : rule11,
        DUE_AT_CLOSING : rule7
    };
}
%}

ROOT -> rule0 
{% ([ rule0 ]) => {
    return {
        
        
    };
}
%}



rule1 -> String {% id %} # BUYER 


rule2 -> " agrees to pay to " 


rule3 -> String {% id %} # SELLER 


rule4 -> " the total sum e" 


rule5 -> Double {% id %} # INITIAL_DUE 


rule6 -> ", in the manner following:\n\nE" 


rule7 -> Double {% id %} # DUE_AT_CLOSING 


rule8 -> " is to be paid at closing, and the remaining balance of E" 


rule9 -> Double {% id %} # TOTAL_DUE_BEFORE_CLOSING 


rule10 -> " shall be paid as follows:\n\nE" 


rule11 -> Double {% id %} # MIN_PAYMENT 


rule12 -> " or more per month on the first day of each and every month, and continuing until the entire balance, including both principal and interest, shall be paid in full -- provided, however, that the entire balance due plus accrued interest and any other amounts due here-under shall be paid in full on or before 24 months.\n\nMonthly payments shall include both principal and interest with interest at the rate of " 


rule13 -> Double {% id %} # INTEREST_RATE 


rule14 -> "%, computed monthly on the remaining balance from time to time unpaid.\n" 


ContractState -> String  __  String 

{% ( data ) => {
    return {
        $class: "org.accordproject.common.ContractState",
        stateId : data[0],
        contract : data[2]
    };
}
%}


ContractModel -> String 

{% ( data ) => {
    return {
        $class: "org.accordproject.common.ContractModel",
        contractId : data[0]
    };
}
%}


ClauseModel -> String 

{% ( data ) => {
    return {
        $class: "org.accordproject.common.ClauseModel",
        clauseId : data[0]
    };
}
%}


Request -> String  __  DateTime 

{% ( data ) => {
    return {
        $class: "org.accordproject.common.Request",
        transactionId : data[0],
        timestamp : data[2]
    };
}
%}


Response -> String  __  DateTime 

{% ( data ) => {
    return {
        $class: "org.accordproject.common.Response",
        transactionId : data[0],
        timestamp : data[2]
    };
}
%}


ErrorResponse -> String  __  DateTime 

{% ( data ) => {
    return {
        $class: "org.accordproject.common.ErrorResponse",
        transactionId : data[0],
        timestamp : data[2]
    };
}
%}


Payload -> ContractModel  __  Request  __  ContractState 

{% ( data ) => {
    return {
        $class: "org.accordproject.common.Payload",
        contract : data[0],
        request : data[2],
        state : data[4]
    };
}
%}


Success -> Response  __  ContractState  __  Event:+ 

{% ( data ) => {
    return {
        $class: "org.accordproject.common.Success",
        response : data[0],
        state : data[2],
        emit : data[4]
    };
}
%}


Failure -> ErrorResponse 

{% ( data ) => {
    return {
        $class: "org.accordproject.common.Failure",
        error : data[0]
    };
}
%}


Installment -> Double  __  String  __  DateTime 

{% ( data ) => {
    return {
        $class: "org.accordproject.installmentsale.Installment",
        amount : data[0],
        transactionId : data[2],
        timestamp : data[4]
    };
}
%}


ClosingPayment -> Double  __  String  __  DateTime 

{% ( data ) => {
    return {
        $class: "org.accordproject.installmentsale.ClosingPayment",
        amount : data[0],
        transactionId : data[2],
        timestamp : data[4]
    };
}
%}


Balance -> Double  __  Double  __  String  __  DateTime 

{% ( data ) => {
    return {
        $class: "org.accordproject.installmentsale.Balance",
        balance : data[0],
        total_paid : data[2],
        transactionId : data[4],
        timestamp : data[6]
    };
}
%}


ContractStatus -> "WaitingForFirstDayOfNextMonth" {% id %} | "Fulfilled" {% id %} 


InstallmentSaleState -> ContractStatus  __  Double  __  Integer  __  Double  __  String  __  String 

{% ( data ) => {
    return {
        $class: "org.accordproject.installmentsale.InstallmentSaleState",
        status : data[0],
        balance_remaining : data[2],
        next_payment_month : data[4],
        total_paid : data[6],
        stateId : data[8],
        contract : data[10]
    };
}
%}


PaymentObligation -> String  __  String  __  Double  __  String  __  DateTime 

{% ( data ) => {
    return {
        $class: "org.accordproject.installmentsale.PaymentObligation",
        from : data[0],
        to : data[2],
        amount : data[4],
        eventId : data[6],
        timestamp : data[8]
    };
}
%}



# Basic types
NUMBER -> [0-9] 
{% (d) => {return parseInt(d[0]);}%}

DOUBLE_NUMBER -> NUMBER NUMBER
{% (d) => {return '' + d[0] + d[1]}%}

MONTH -> DOUBLE_NUMBER
DAY -> DOUBLE_NUMBER
YEAR -> DOUBLE_NUMBER DOUBLE_NUMBER
{% (d) => {return '' + d[0] + d[1]}%}

DATE -> MONTH "/" DAY "/" YEAR
{% (d) => {return '' + d[4] + '-' + d[0] + '-' + d[2]}%}

Word -> [\S]:*
{% (d) => {return d[0].join('');}%}

BRACKET_PHRASE -> "[" Word (__ Word):* "]" {% ((d) => {return d[1] + ' ' + flatten(d[2]).join(" ");}) %}

String -> dqstring {% id %}
Double -> decimal {% id %}
Integer -> int {% id %}
Long -> int {% id %}
Boolean -> "true" {% id %} | "false" {% id %}
DateTime -> DATE  {% id %}